import requests
import datetime
import regex as re
from bs4 import BeautifulSoup
from urllib.parse import urlparse

from threatingestor.sources import Source
import threatingestor.artifacts

class Plugin(Source):

    def __init__(self, name, url, filter=None, path=None):
        self.name = name
        self.url = url
        self.filter = filter
        self.path = path

    def run(self, saved_state):
        # Configures sitemap parsing
        response = requests.get(self.url)
        xml = BeautifulSoup(response.text, "lxml-xml")

        urls = xml.find_all("url")
        artifacts_list = []

        for u in urls:

            # Extracts only the 'loc' tag from the xml
            if xml.find("loc"):
                loc = u.findNext("loc").text
            else:
                loc = ""

            if self.filter is not None:
                # Regex input via config.yml
                # Example: security|threat|malware
                xml_query = re.compile(r"{0}".format(self.filter)).findall(str(self.filter.split('|')))

                # Iterates over the regex output to locate all provided keywords
                for x in xml_query:
                    # Uses a path instead of a keyword
                    if self.path is not None:
                        if self.path in loc:
                            artifacts_list.append(loc)
                    
                    # Only filters using a keyword
                    if self.path is None:
                        if x in loc:
                            artifacts_list.append(loc)
            
            elif self.filter is None and self.path is not None:
                # Filters only by path in XML loc, no set filter
                # Default: /path/name/*

                if self.path in loc:
                    artifacts_list.append(loc)
            
            else:
                # Locates all blog links within the sitemap
                if "blog" in loc:
                    artifacts_list.append(loc)

        artifacts = []

        for a in artifacts_list:
            description = 'URL: {u}\nTask autogenerated by ThreatIngestor from source: {s}'.format(u=a, s=self.name)
            artifact = threatingestor.artifacts.URL(a, self.name, reference_link=self.url, reference_text=description)
            artifacts.append(artifact)

        # print(artifacts)
        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + "Z"

        return saved_state, artifacts



import unittest
from unittest.mock import Mock, patch
import json

import httpretty

import threatingestor.sources.sqs
import threatingestor.artifacts
import threatingestor.exceptions


class TestSQS(unittest.TestCase):

    @patch('boto3.resource')
    def setUp(self, boto3_resource):
        self.sqs = threatingestor.sources.sqs.Plugin('a', 'b', 'c', 'd', 'e', ['link'])

    def test_run_reads_messages_deletes_returns_artifacts(self):
        message = Mock()
        message.body = '{"link": "http://example.mock/path"}'
        self.sqs.queue.receive_messages.return_value = [message]

        saved_state, artifact_list = self.sqs.run(None)

        self.assertEqual(saved_state, None)
        self.sqs.queue.receive_messages.assert_called_once_with()
        message.delete.assert_called_once_with()
        self.assertIn('http://example.mock/path', [str(x) for x in artifact_list])



from __future__ import absolute_import

import csv

import threatingestor.artifacts
from threatingestor.operators import Operator

class Plugin(Operator):
    """Operator for output to flat CSV file."""
    def __init__(self, filename, artifact_types=None, filter_string=None, allowed_sources=None):
        """CSV operator."""
        self.filename = filename

        super(Plugin, self).__init__(artifact_types, filter_string, allowed_sources)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
        ]


    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        with open(self.filename, 'a+', encoding='utf-8') as f:
            writer = csv.writer(f)
            artifact_type = artifact.__class__.__name__
            writer.writerow([artifact_type, str(artifact),
                    artifact.reference_link,
                    artifact.reference_text])



import unittest
from unittest.mock import patch, Mock

import threatingestor.operators.twitter
import threatingestor.artifacts
import threatingestor.exceptions


class TestTwitter(unittest.TestCase):

    @patch('twitter.Twitter')
    def setUp(self, Twitter):
        self.twitter = threatingestor.operators.twitter.Plugin('a', 'b', 'c', 'd', 'status: {artifact}')

    def test_handle_artifact_interpolates_status_message(self):
        self.twitter._tweet = Mock()
        artifact = threatingestor.artifacts.URL('http://somedomain.com/test', 'test')
        expected_content = 'status: http://somedomain.com/test'
        self.twitter.handle_artifact(artifact)
        self.twitter._tweet.assert_called_once_with(expected_content, quote_tweet=None)

    def test_handle_artifact_passes_quote_tweet_for_tweet_links(self):
        self.twitter._tweet = Mock()
        artifact = threatingestor.artifacts.URL('http://somedomain.com/test', 'test',
                                                'https://twitter.com/InQuest/status/00000000000')
        expected_content = 'status: http://somedomain.com/test'
        self.twitter.handle_artifact(artifact)
        self.twitter._tweet.assert_called_once_with(expected_content,
                                                    quote_tweet='https://twitter.com/InQuest/status/00000000000')

        # make sure quote_tweet doesn't get set if the link isn't a tweet permalink
        self.twitter._tweet.reset_mock()
        artifact = threatingestor.artifacts.URL('http://somedomain.com/test', 'test',
                                                'https://twitter.com/help/')
        expected_content = 'status: http://somedomain.com/test'
        self.twitter.handle_artifact(artifact)
        self.twitter._tweet.assert_called_once_with(expected_content,
                                                    quote_tweet=None)

    def test_artifact_types_are_set_if_passed_in_else_default(self):
        artifact_types = [
            threatingestor.artifacts.URL,
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
        ]
        self.assertEqual(threatingestor.operators.twitter.Plugin('a', 'b', 'c', 'd', 'e',
                                                                  artifact_types=[threatingestor.artifacts.URL]).artifact_types,
                          [threatingestor.artifacts.URL])
        self.assertEqual(threatingestor.operators.twitter.Plugin('a', 'b', 'c', 'd', 'e').artifact_types,
                          artifact_types)

    def test_init_sets_config_args(self):
        operator = threatingestor.operators.twitter.Plugin('a', 'b', 'c', 'd', 'e',
                                                           filter_string='test',
                                                           allowed_sources=['test-one'])
        self.assertEqual(operator.filter_string, 'test')
        self.assertEqual(operator.allowed_sources, ['test-one'])

    def test_init_raises_if_bad_status(self):
        with self.assertRaises(threatingestor.exceptions.IngestorError):
            operator = threatingestor.operators.twitter.Plugin('a', 'b', 'c', 'd',
                                                               filter_string='test',
                                                               allowed_sources=['test-one'],
                                                               status=[])



import unittest
from unittest.mock import patch, ANY as MOCK_ANY

import threatingestor.operators.csv
import threatingestor.artifacts


class TestCSV(unittest.TestCase):

    def setUp(self):
        self.csv = threatingestor.operators.csv.Plugin('a')

    def test_artifact_types_are_set_if_passed_in_else_default(self):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        self.assertEqual(threatingestor.operators.csv.Plugin('a', artifact_types=artifact_types).artifact_types, artifact_types)
        self.assertEqual(threatingestor.operators.csv.Plugin('a').artifact_types, [threatingestor.artifacts.Domain, threatingestor.artifacts.Hash, threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL])

    def test_filter_list_and_allowed_sources_are_set_if_passed_in(self):
        self.assertEqual(threatingestor.operators.csv.Plugin('a', filter_string='test').filter_string, 'test')
        self.assertEqual(threatingestor.operators.csv.Plugin('a', allowed_sources=['test-one']).allowed_sources, ['test-one'])



import requests

from threatingestor.sources import Source

class Plugin(Source):

    def __init__(self, name, url):
        self.name = name
        self.url = url


    def run(self, saved_state):
        # Read saved state and set HTTP headers.
        headers = {}
        
        if saved_state:
            # If both last modified and etag, set both.
            # Otherwise just interpret the whole field as last modified.
            last_modified = saved_state

            if len(saved_state.split(';')) == 2:
                last_modified, etag = saved_state.split(';')
                headers['If-None-Match'] = etag

            headers['If-Modified-Since'] = last_modified
        
        if not saved_state:
            saved_state = ""

        # Send head first to check 304.
        response = requests.head(self.url, headers=headers)

        # If not modified, return immediately
        if "304" in saved_state or response.status_code == 304:
            saved_state = ';'.join([str(response.headers.get('Last-Modified')), 'N/A', str(response.status_code)])
            return saved_state, []

        # Otherwise, do the full request.
        response = requests.get(self.url, headers=headers)

        # Form saved state.
        last_modified = response.headers.get('Last-Modified')
        etag = response.headers.get('Etag')

        if etag:
            saved_state = ';'.join([str(last_modified), etag, str(response.status_code)])
        else:
            saved_state = last_modified

        # Process text.
        artifact_list = self.process_element(response.text, self.url, include_nonobfuscated=True)

        return saved_state, artifact_list



import re
from urllib.parse import urlparse
import itertools
import iocextract
from loguru import logger

import threatingestor.artifacts

TRUNCATE_LENGTH = 140

class Source:
    """Base class for all Source plugins.

    Note: This is an abstract class. You must override ``__init__`` and ``run``
    in child classes. You should not override ``process_element``. When adding
    additional methods to child classes, consider prefixing the method name
    with an underscore to denote a ``_private_method``.
    """
    def __init__(self, name, *args, **kwargs):
        """Override this constructor in child classes.

        The first argument must always be ``name``.

        Other argumentss should be url, auth, etc, whatever is needed to set
        up the object.
        """
        raise NotImplementedError()


    def run(self, saved_state):
        """Run and return ``(saved_state, list(Artifact))``.

        Override this method in child classes.

        The method signature and return values must remain consistent.

        The method should attempt to pick up where we left off using
        ``saved_state``, if supported. If ``saved_state`` is ``None``, you can
        assume this is a first run. If state is maintained by the remote
        resource (e.g. as it is with SQS), ``saved_state`` should always be
        ``None``.
        """
        raise NotImplementedError()


    def process_element(self, content, reference_link, include_nonobfuscated=False):
        """Take a single source content/url and return a list of Artifacts.

        This is the main work block of Source plugins, which handles
        IOC extraction and artifact creation.

        :param content: String content to extract from.
        :param reference_link: Reference link to attach to all artifacts.
        :param include_nonobfuscated: Include non-defanged URLs in output?
        """
        logger.debug(f"Processing in source '{self.name}'")

        # Truncate content to a reasonable length for reference_text.
        reference_text = content[:TRUNCATE_LENGTH] + ('...' if len(content) > TRUNCATE_LENGTH else '')

        # Initialize an empty list and a map of counters to track each artifact type.
        artifact_list = []
        artifact_type_count = {
            'domain': 0,
            'hash': 0,
            'ipaddress': 0,
            'task': 0,
            'url': 0,
            'yarasignature': 0,
            'email': 0,
        }

        # Collect URLs and domains.
        scraped = itertools.chain(
            iocextract.extract_unencoded_urls(content),
            # Decode encoded URLs, since we can't operate on encoded ones.
            iocextract.extract_encoded_urls(content, refang=True),
        )
        for url in scraped:
            # Dump anything with ellipses, these get through the regex.
            if u'\u2026' in url:
                continue

            artifact = threatingestor.artifacts.URL(url, self.name,
                                                    reference_link=reference_link,
                                                    reference_text=reference_text)

            # Dump URLs that appear to have the same domain as reference_url.
            try:
                if artifact.domain() == urlparse(reference_link).netloc:
                    continue
            except ValueError:
                # Error parsing reference_link as a URL. Ignoring.
                pass

            if artifact.is_obfuscated() or include_nonobfuscated:
                # Do URL collection.
                artifact_list.append(artifact)
                artifact_type_count['url'] += 1

                # Do domain collection in the same pass.
                # Note: domains will always be a subset of URLs. There is no
                # domain extraction.
                if artifact.is_domain():
                    artifact_list.append(
                            threatingestor.artifacts.Domain(artifact.domain(), self.name,
                                                            reference_link=reference_link,
                                                            reference_text=reference_text))
                    artifact_type_count['domain'] += 1

        # Collect IPs.
        scraped = iocextract.extract_ips(content)
        for ip in scraped:
            artifact = threatingestor.artifacts.IPAddress(ip, self.name,
                                                          reference_link=reference_link,
                                                          reference_text=reference_text)

            try:
                ipaddress = artifact.ipaddress()
                if ipaddress.is_private or ipaddress.is_loopback or ipaddress.is_reserved:
                    # Skip private, loopback, reserved IPs.
                    continue

            except ValueError:
                # Skip invalid IPs.
                continue

            artifact_list.append(artifact)
            artifact_type_count['ipaddress'] += 1

        # Collect YARA rules.
        scraped = iocextract.extract_yara_rules(content)
        for rule in scraped:
            artifact = threatingestor.artifacts.YARASignature(rule, self.name,
                                                              reference_link=reference_link,
                                                              reference_text=reference_text)

            artifact_list.append(artifact)
            artifact_type_count['yarasignature'] += 1

        # Collect hashes.
        scraped = iocextract.extract_hashes(content)
        for hash_ in scraped:
            artifact = threatingestor.artifacts.Hash(hash_, self.name,
                                                     reference_link=reference_link,
                                                     reference_text=reference_text)

            artifact_list.append(artifact)
            artifact_type_count['hash'] += 1

        # Collect emails.
        scraped = itertools.chain(iocextract.extract_emails(content))

        for email in scraped:
            artifact_list.append(threatingestor.artifacts.Email(email, self.name, reference_link=reference_link, reference_text=reference_text))
            
            artifact_type_count['email'] += 1

        # Generate generic task.
        title = f"Manual Task: {reference_link}"
        description = f"URL: {reference_link}\nTask autogenerated by ThreatIngestor from source: {self.name}"
        artifact = threatingestor.artifacts.Task(title, self.name,
                                                 reference_link=reference_link,
                                                 reference_text=description)
        artifact_list.append(artifact)
        artifact_type_count['task'] += 1

        logger.debug(f"Found {len(artifact_list)} total artifacts")
        logger.debug(f"Type breakdown: {artifact_type_count}")
        return artifact_list



import unittest
from unittest.mock import patch
import datetime

import responses

import threatingestor.sources.github

API_RESPONSE_DATA = """
{
  "total_count": 40,
  "incomplete_results": false,
  "items": [
    {
      "id": 3081286,
      "name": "Tetris",
      "full_name": "dtrupenn/Tetris",
      "owner": {
        "login": "dtrupenn",
        "id": 872147,
        "avatar_url": "https://example.com/a.png",
        "gravatar_id": "",
        "url": "https://api.github.com/users/dtrupenn",
        "received_events_url": "https://api.github.com/users/dtrupenn/received_events",
        "type": "User"
      },
      "private": false,
      "html_url": "https://github.com/dtrupenn/Tetris",
      "description": "A C implementation of Tetris using Pennsim through LC4",
      "fork": false,
      "url": "https://api.github.com/repos/dtrupenn/Tetris",
      "created_at": "2012-01-01T00:31:50Z",
      "updated_at": "2013-01-05T17:58:47Z",
      "pushed_at": "2012-01-01T00:37:02Z",
      "homepage": "",
      "size": 524,
      "stargazers_count": 1,
      "watchers_count": 1,
      "language": "Assembly",
      "forks_count": 0,
      "open_issues_count": 0,
      "master_branch": "master",
      "default_branch": "master",
      "score": 10.309712
    }
  ]
}
"""


class TestGitHub(unittest.TestCase):

    def setUp(self):
        self.github = threatingestor.sources.github.Plugin('mygithub', 'CVE-2018-')

    @patch('threatingestor.sources.github.datetime')
    @responses.activate
    def test_run_returns_saved_state_tasks(self, mock_datetime):
        responses.add(responses.GET, threatingestor.sources.github.REPO_SEARCH_URL,
                body=API_RESPONSE_DATA)
        mock_datetime.datetime.utcnow.return_value = datetime.datetime(2018, 4, 30, 17, 5, 13, 194840)
        mock_datetime.datetime.side_effect = lambda *args, **kw: datetime.datetime(*args, **kw)

        saved_state, artifact_list = self.github.run(None)
        self.assertIn('Manual Task: GitHub dtrupenn/Tetris', [str(x) for x in artifact_list])
        self.assertEqual(len(artifact_list), 1)
        self.assertEqual(saved_state, '2018-04-30T17:05:13Z')

    @responses.activate
    def test_requests_until_rel_is_not_next(self):
        responses.add(responses.GET,
                "https://api.github.com/search/repositories?q=Test",
                match_querystring=True, body=API_RESPONSE_DATA,
                headers={"Link":'<https://api.github.com/search/repositories?q=Test&page=2>; rel="next"'})
        responses.add(responses.GET,
                'https://api.github.com/search/repositories?q=Test&page=2',
                match_querystring=True, body=API_RESPONSE_DATA,
                headers={"Link":'<https://api.github.com/search/repositories?q=Test&page=3>; rel="next"'})
        responses.add(responses.GET,
                'https://api.github.com/search/repositories?q=Test&page=3',
                match_querystring=True, body=API_RESPONSE_DATA,
                headers={"Link":'<https://api.github.com/search/repositories?q=Test&page=3>; rel="last"'})

        params = {"q":"Test"}
        repo_list = self.github._repository_search(params)
        self.assertIn('Tetris', [str(x['name']) for x in repo_list])
        self.assertEqual(len(repo_list), 3)



import os
import sys
import string
import random
import requests
import datetime
import iocextract
import numpy as np
from loguru import logger

try:
    import numpy as np
except ImportError:
    logger.error("Missing the following package(s): numpy")
    sys.exit()

try:
    import cv2
except ImportError:
    logger.error("Missing the following package(s): opencv-python")
    sys.exit()

try:
    import pytesseract
except ImportError:
    logger.error("Missing the following package(s): pytesseract")
    sys.exit()

from threatingestor.sources import Source
import threatingestor.artifacts

# Creates a random string with a length of 5
def tmp_name():
    return "".join(random.choice(string.ascii_lowercase) for _ in range(5))

tmp_file = str(tmp_name())

class Plugin(Source):
    """
    Image text extraction using Google's OCR Tesseract engine and computer vision
    """

    def __init__(self, name, img=""):
        self.name = name
        self.img = img

        if "http" in img:
            with open(f"/tmp/{tmp_file}", "wb") as i:
                i.write(requests.get(str(self.img)).content)

    def run(self, saved_state):
        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + "Z"

        if os.path.exists(f"/tmp/{tmp_file}"):
            data = cv2.imread(f"/tmp/{tmp_file}")
        else:
            # No image is present
            try:
                data = cv2.imread(self.img)
            except TypeError:
                pass

        try:
            # Helps with preprocessing by converting to a grayscale
            grayscale_img = cv2.cvtColor(data, cv2.COLOR_BGR2GRAY)

            # Creates a binary image by using the proper threshold from cv and inverts the binary
            invert_img = cv2.bitwise_not(cv2.threshold(grayscale_img, 130, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)[1])

            # Helps with cleanup
            process_iter = cv2.dilate(cv2.erode(invert_img, np.ones((2,2), np.uint8), iterations=1), np.ones((2,2), np.uint8), iterations=1)

            # Converts image data to a string
            img_data = pytesseract.image_to_string(process_iter)

            artifact_list = []

            title = "Image: {0}".format(self.img)
            description = 'URL: {u}\nTask autogenerated by ThreatIngestor from source: {s}'
            description = description.format(s=self.name, u=list(iocextract.extract_urls(img_data)))
            artifact = threatingestor.artifacts.Task(title, self.name, reference_link=str(list(iocextract.extract_urls(img_data))), reference_text=description)
            artifact_list.append(artifact)

            os.remove(f"/tmp/{tmp_file}")
                
        except cv2.error:
            raise FileNotFoundError

        return saved_state, artifact_list


import json

from pathlib import Path
from typing import List, Set
from loguru import logger

class Whitelist:
    """Base class for Whitelist plugin."""

    def __init__(self, paths: List[str]):
        self.paths = paths
        self.values: Set[str] = set()
        self._load_paths()

    def contains(self, value: str) -> bool:
        return value in self.values

    def _load_path(self, path: str):
        if Path(path).is_file():
            try:
                with open(path) as f:
                    data = json.load(f)
                    list_ = data.get("list", [])
                    self.values.update(list_)
            except json.decoder.JSONDecodeError:
                logger.exception(f"Couldn't load {path} as a JSON file")


    def _load_paths(self):
        for path in self.paths:
            self._load_path(path)



import sys
import time
import collections

from loguru import logger
import statsd

try:
    import notifiers
    from notifiers.logging import NotificationHandler
except ImportError:
    logger.info("Notifiers is not installed.")
    notifiers = None

import threatingestor.config
import threatingestor.state
import threatingestor.exceptions
import threatingestor.whitelist

class Ingestor:
    """ThreatIngestor main work logic.

    Handles reading the config file, calling sources, maintaining state, and
    sending artifacts to operators.
    """
    def __init__(self, config_file):
        # Load config.
        try:
            logger.debug(f"Reading config from '{config_file}'")
            self.config = config.Config(config_file)
        except (OSError, threatingestor.exceptions.IngestorError):
            # Error loading config.
            logger.exception("Couldn't read config")
            sys.exit(1)

        # Configure logging with optional notifiers.
        logger.configure(**self.config.logging())
        try:
            logger.level("NOTIFY", no=35, color="<yellow>", icon="\U0001F514")
        except TypeError:
            # logger raises TypeError if NOTIFY is already defined
            pass

        if notifiers:
            notifier_config = self.config.notifiers()
            notifier = notifiers.get_notifier(notifier_config.get('provider'))

            if notifier:
                logger.debug(f"Adding notification handler '{notifier_config.get('provider')}'")
                # Notifier 'provider_name' is set and valid.
                handler = NotificationHandler(**notifier_config)
                logger.add(handler, level="NOTIFY")

        logger.debug("Log handler reconfigured")

        # Configure statsd.
        try:
            self.statsd = statsd.StatsClient(**self.config.statsd())
            self.statsd.incr('start')
        except TypeError:
            logger.exception("Couldn't initialize statsd client; bad config?")
            sys.exit(1)

        # Load state DB.
        try:
            logger.debug(f"Opening state database '{self.config.state_path()}'")
            self.statedb = threatingestor.state.State(self.config.state_path())
        except (OSError, IOError, threatingestor.exceptions.IngestorError):
            # Error loading state DB.
            logger.exception("Error reading state database")
            sys.exit(1)

        # Instantiate plugins.
        try:
            logger.debug("Initializing sources")
            self.sources = {name: source(**kwargs)
                            for name, source, kwargs in self.config.sources()}

            logger.debug("Initializing operators")
            self.operators = {name: operator(**kwargs)
                              for name, operator, kwargs in self.config.operators()}

            logger.debug("Initializing whitelists")
            self.whitelist = threatingestor.whitelist.Whitelist(self.config.whitelists())

        except (TypeError, ConnectionError, threatingestor.exceptions.PluginError):
            logger.warning("Twitter config format has recently changed. See https://github.com/InQuest/ThreatIngestor/releases/tag/v1.0.0b5")
            logger.exception("Error initializing plugins")
            sys.exit(1)

    def _is_whitelisted(self, artifact) -> bool:
        if self.whitelist.contains(str(artifact)):
            logger.debug(
                f"Reject {str(artifact)} from further processing because it is whitelisted."
            )
            return True
        return False

    def run(self):
        """Run once, or forever, depending on config."""
        if self.config.daemon():
            logger.debug("Running forever, in a loop")
            self.run_forever()
        else:
            logger.debug("Running once, to completion")
            with self.statsd.timer('run_once'):
                self.run_once()


    def run_once(self):
        """Run each source once, passing artifacts to each operator."""
        # Track some statistics about artifacts in a summary object.
        summary = collections.Counter()

        for source in self.sources:
            # Run the source to collect artifacts.
            logger.debug(f"Running source '{source}'")
            try:
                with self.statsd.timer(f'source.{source}'):
                    saved_state, artifacts = self.sources[source].run(self.statedb.get_state(source))

            except Exception:
                self.statsd.incr(f'error.source.{source}')
                logger.exception(f"Unknown error in source '{source}'")
                continue

            # Save the source state.
            self.statedb.save_state(source, saved_state)

            # Reject whitelisted artifacts
            artifacts = [
                artifact
                for artifact in artifacts
                if not self._is_whitelisted(artifact)
            ]

            # Process artifacts with each operator.
            for operator in self.operators:
                logger.debug(f"Processing {len(artifacts)} artifacts from source '{source}' with operator '{operator}'")
                try:
                    with self.statsd.timer(f'operator.{operator}'):
                        self.operators[operator].process(artifacts)

                except Exception:
                    self.statsd.incr(f'error.operator.{operator}')
                    logger.exception(f"Unknown error in operator '{operator}'")
                    continue

            # Record stats and update the summary.
            types = artifact_types(artifacts)
            summary.update(types)
            for artifact_type in types:
                self.statsd.incr(f'source.{source}.{artifact_type}', types[artifact_type])
                self.statsd.incr(f'artifacts.{artifact_type}', types[artifact_type])

        # Log the summary.
        logger.log('NOTIFY', f"New artifacts: {dict(summary)}")


    def run_forever(self):
        """Run forever, sleeping for the configured interval between each run."""
        while True:
            with self.statsd.timer('run_once'):
                self.run_once()

            logger.debug(f"Sleeping for {self.config.sleep()} seconds")
            time.sleep(self.config.sleep())


def artifact_types(artifact_list):
    """Return a dictionary with counts of each artifact type."""
    types = {}
    for artifact in artifact_list:
        artifact_type = artifact.__class__.__name__.lower()
        if artifact_type in types:
            types[artifact_type] += 1
        else:
            types[artifact_type] = 1

    return types


def main():
    """CLI entry point, uses sys.argv directly."""
    if len(sys.argv) < 2:
        logger.error("You must specify a config file")
        sys.exit(1)

    app = Ingestor(sys.argv[1])
    app.run()


if __name__ == "__main__":
    main()



import unittest
from unittest.mock import patch

import threatingestor.sources.twitter


class TestTwitter(unittest.TestCase):

    @patch('twitter.Twitter')
    def setUp(self, Twitter):
        self.twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e')

    def teadDown(self):
        self.twitter.api.reset_mock()

    @patch('twitter.Twitter')
    def test_init_detects_search_type(self, Twitter):
        # default to @mentions
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e')
        self.assertEqual(twitter.endpoint._mock_name, 'mentions_timeline')
        self.assertEqual(twitter.name, 'a')

        # slug and owner_screen_name => list
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', slug='test', owner_screen_name='test')
        self.assertEqual(twitter.endpoint._mock_name, 'statuses')
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', slug='test')
        self.assertEqual(twitter.endpoint._mock_name, 'mentions_timeline')
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', owner_screen_name='test')
        self.assertEqual(twitter.endpoint._mock_name, 'mentions_timeline')

        # screen_name or user_id => user
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', screen_name='test')
        self.assertEqual(twitter.endpoint._mock_name, 'user_timeline')
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', user_id=1)
        self.assertEqual(twitter.endpoint._mock_name, 'user_timeline')

        # q => search
        twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', q='test')
        self.assertEqual(twitter.endpoint._mock_name, 'tweets')
        self.assertEqual(twitter.name, 'a')

    def test_run_respects_saved_state(self):
        self.twitter.run('12345')
        self.twitter.endpoint.assert_called_once_with(tweet_mode='extended', since_id='12345')

    def test_run_returns_newest_tweet_id_as_saved_state(self):
        self.twitter.endpoint.return_value = [
            {
                'text': 'test',
                'id_str': '12346',
                'user': {'screen_name': 'test'},
            },
            {
                'text': 'test',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
            },
            {
                'text': 'test',
                'id_str': '12344',
                'user': {'screen_name': 'test'},
            },
        ]
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(saved_state, '12346')

    def test_run_supports_all_endpoints(self):
        # both supported return formats
        self.twitter.endpoint.return_value = [
            {
                'text': 'test',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
            },
        ]
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(saved_state, '12345')

        self.twitter.endpoint.return_value = {
            'statuses': [
                {
                    'text': 'test',
                    'id_str': '12345',
                    'user': {'screen_name': 'test'},
                },
            ]
        }
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(saved_state, '12345')

    def test_run_returns_artifacts_correctly(self):
        self.twitter.endpoint.return_value = [
            {
                'full_text': 'hxxp://someurl.com/test',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
            },
        ]
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(len(artifact_list), 3)
        self.assertIn('someurl.com', [str(x) for x in artifact_list])
        self.assertIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertEqual('https://twitter.com/test/status/12345', artifact_list[0].reference_link)

    def test_run_expands_tco_links_if_available(self):
        self.twitter.endpoint.return_value = [
            {
                'full_text': 'https://t.co/t3s7',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
                'entities': {
                    'urls': [
                        {
                            'url': 'https://t.co/t3s7',
                            'expanded_url': 'hxxp://someurl.com/test',
                        },
                    ],
                },
            },
        ]
        # note: using hxxp above just to get the expanded url to be processed as
        # an obfuscated url and added to the threatingestor.artifacts list
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(len(artifact_list), 3)
        self.assertIn('someurl.com', [str(x) for x in artifact_list])
        self.assertIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertNotIn('t.co', [str(x) for x in artifact_list])
        self.assertNotIn('https://t.co/t3s7', [str(x) for x in artifact_list])

    @patch('twitter.Twitter')
    def test_run_includes_nonobfuscated_iff_configured(self, Twitter):
        # control
        self.twitter.endpoint.return_value = [
            {
                'full_text': 'http://someurl.com/test',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
            },
        ]
        saved_state, artifact_list = self.twitter.run(None)
        self.assertEqual(len(artifact_list), 1)

        # test with defanged_only=False
        new_twitter = threatingestor.sources.twitter.Plugin('a', 'b', 'c', 'd', 'e', defanged_only=False)
        new_twitter.endpoint.return_value = [
            {
                'full_text': 'http://someurl.com/test',
                'id_str': '12345',
                'user': {'screen_name': 'test'},
            },
        ]
        saved_state, artifact_list = new_twitter.run(None)
        self.assertEqual(len(artifact_list), 3)

        self.assertIn('someurl.com', [str(x) for x in artifact_list])
        self.assertIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertEqual('https://twitter.com/test/status/12345', artifact_list[0].reference_link)



from __future__ import absolute_import

import os
import random
import string
import twitter
import requests
from loguru import logger
from pyshorteners import Shortener, exceptions

try:
    import numpy as np
    installed_np = True
except ImportError:
    logger.warning("Missing the following package(s): numpy")
    installed_np = False

try:
    import cv2
    installed_cv = True
except ImportError:
    logger.warning("Missing the following package(s): opencv-python")
    installed_cv = False

try:
    import pytesseract
    installed_tesseract = True
except ImportError:
    logger.warning("Missing the following package(s): pytesseract")
    installed_tesseract = False

from threatingestor.sources import Source

TWEET_URL = 'https://twitter.com/{user}/status/{id}'

s = Shortener()

# Creates a random string with a length of 5
def tmp_name():
    return "".join(random.choice(string.ascii_lowercase) for _ in range(5))

class Plugin(Source):

    def __init__(self, name, api_key, api_secret_key, access_token, access_token_secret, defanged_only=True, **kwargs):
        self.name = name
        self.api = twitter.Twitter(auth=twitter.OAuth(access_token, access_token_secret, api_key, api_secret_key))

        # Let the user decide whether to include non-obfuscated URLs or not.
        self.include_nonobfuscated = not defanged_only

        # Support for full tweet
        tweet_param = {'tweet_mode': 'extended'}
        kwargs.update(tweet_param)

        # Forward kwargs.
        # NOTE: No validation is done here, so if the config is wrong, expect bad results.
        self.kwargs = kwargs

        # Decide which endpoint to use based on passed arguments.
        # If slug and owner_screen_name, use List API.
        # If screen_name or user_id, use User Timeline API.
        # If q is set, use Search API.
        # Otherwise, default to mentions API.
        self.endpoint = self.api.statuses.mentions_timeline

        if (kwargs.get('slug') and kwargs.get('owner_screen_name')) or (kwargs.get('list_id') and kwargs.get('owner_screen_name')):
            self.endpoint = self.api.lists.statuses
        elif kwargs.get('screen_name') or kwargs.get('user_id'):
            self.endpoint = self.api.statuses.user_timeline
        elif kwargs.get('q'):
            self.endpoint = self.api.search.tweets

    def run(self, saved_state):
        # Modify kwargs to insert since_id.
        if saved_state:
            self.kwargs['since_id'] = saved_state

        # Pull new tweets.
        try:
            response = self.endpoint(**self.kwargs)
        except twitter.api.TwitterHTTPError as e:
            # API error; log and return early.
            logger.warning(f"Twitter API Error: {e}")

            return saved_state, []

        # Correctly handle responses from different endpoints.
        try:
            tweet_list = response['statuses']
        except TypeError:
            tweet_list = response

        tweets = []
        
        for tweet in tweet_list:
            if "retweeted_status" in tweet:
                content = tweet['retweeted_status'].get('full_text', '')
            else:
                content = tweet.get('full_text', '')

            tweets.append({
                'content': content,
                'id': tweet.get('id_str', ''),
                'user': tweet.get('user', {}).get('screen_name', ''),
                'entities': tweet.get('entities', {}),
            })

        artifacts = []

        # Traverse in reverse, old to new.
        tweets.reverse()

        tmp_file = str(tmp_name())

        for tweet in tweets:

            # Expand t.co links.
            for url in tweet['entities'].get('urls', []):
                try:
                    tweet['content'] = tweet['content'].replace(url['url'], url['expanded_url'])
                except KeyError:
                    
                    # Attempts to expand the URL if not available through Twitter
                    try:
                        tweet['content'] = tweet['content'].replace(url['url'], str(s.tinyurl.expand(url['url'])))
                    except exceptions.ExpandingErrorException:
                        # If unable to expand the URL, this exception is thrown
                        pass

            # Process tweet
            saved_state = tweet['id']
            artifacts += self.process_element(tweet['content'], TWEET_URL.format(user=tweet['user'], id=tweet['id']), include_nonobfuscated=self.include_nonobfuscated)

            if installed_cv and installed_np:

                for media_url in tweet['entities'].get('media', []):
                    img = media_url["media_url_https"]

                    if "http" in img:
                        with open(f"/tmp/{tmp_file}", "wb") as i:
                            i.write(requests.get(str(img)).content)

                    if os.path.exists(f"/tmp/{tmp_file}"):
                        data = cv2.imread(f"/tmp/{tmp_file}")
                    else:
                        # No image is present
                        try:
                            data = cv2.imread(img)
                        except TypeError:
                            pass
                        
                    try:
                        # Creates a binary image by using the proper threshold from cv + converts to grayscale
                        # Inverts the binary
                        invert_img = cv2.bitwise_not(cv2.threshold(cv2.cvtColor(data, cv2.COLOR_BGR2GRAY), 130, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)[1])

                        # Helps with cleanup
                        process_iter = cv2.dilate(cv2.erode(invert_img, np.ones((2,2), np.uint8), iterations=1), np.ones((2,2), np.uint8), iterations=1)

                        if installed_tesseract:
                            # Converts image data to a string
                            img_data = pytesseract.image_to_string(process_iter)
                        else:
                            img_data = ""

                        # Process media
                        saved_state = tweet['id']
                        artifacts += self.process_element(img_data, img, include_nonobfuscated=self.include_nonobfuscated)

                        os.remove(f"/tmp/{tmp_file}")
                            
                    except cv2.error:
                        raise FileNotFoundError

        return saved_state, artifacts



import unittest
from unittest.mock import patch

import threatingestor.operators.mysql
import threatingestor.artifacts


class TestThreatSQLite(unittest.TestCase):

    @patch('pymysql.connect')
    def setUp(self, connect):
        self.mysql = threatingestor.operators.mysql.Plugin('a', 'b', 'c')

    @patch('pymysql.connect')
    def test_artifact_types_are_set_if_passed_in_else_default(self, connect):
        artifact_types = [threatingestor.artifacts.IPAddress, threatingestor.artifacts.URL]
        self.assertEqual(threatingestor.operators.mysql.Plugin('a', 'b', 'c', artifact_types=artifact_types).artifact_types, artifact_types)
        self.assertEqual(threatingestor.operators.mysql.Plugin('a', 'b', 'c').artifact_types, [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature,
            threatingestor.artifacts.Task
        ])

    @patch('pymysql.connect')
    def test_filter_string_and_allowed_sources_are_set_if_passed_in(self, connect):
        self.assertEqual(threatingestor.operators.mysql.Plugin('a', 'b', 'c', filter_string='test').filter_string, 'test')
        self.assertEqual(threatingestor.operators.mysql.Plugin('a', 'b', 'c', allowed_sources=['test-one']).allowed_sources, ['test-one'])



import unittest
import httpretty

import threatingestor.sources.web

class TestWeb(unittest.TestCase):

    def setUp(self):
        self.web = threatingestor.sources.web.Plugin('myweb', 'http://example.mock/list.txt')

    @httpretty.activate
    def test_run_with_200(self):
        httpretty.register_uri(httpretty.HEAD, "http://example.mock/list.txt")
        httpretty.register_uri(httpretty.GET, "http://example.mock/list.txt",
                body='http://example.com/test',
                adding_headers={
                    'Last-Modified': 'test',
                    'Etag': '"test"'
                })

        saved_state, artifacts = self.web.run(None)
        self.assertIn('http://example.com/test', [str(x) for x in artifacts])
        self.assertEqual(saved_state, 'test;"test";200')

    @httpretty.activate
    def test_run_with_304(self):
        httpretty.register_uri(httpretty.HEAD, "http://example.mock/list.txt",
                               status=304)

        saved_state, artifacts = self.web.run('test;"test"')
        self.assertEqual(len(artifacts), 0)

    @httpretty.activate
    def test_run_with_200_and_no_etag(self):
        httpretty.register_uri(httpretty.HEAD, "http://example.mock/list.txt")
        httpretty.register_uri(httpretty.GET, "http://example.mock/list.txt",
                body='http://example.com/test',
                adding_headers={
                    'Last-Modified': 'test',
                })

        saved_state, artifacts = self.web.run(None)
        self.assertIn('http://example.com/test', [str(x) for x in artifacts])
        self.assertEqual(saved_state, 'test')

    @httpretty.activate
    def test_run_with_200_and_no_last_modified(self):
        httpretty.register_uri(httpretty.HEAD, "http://example.mock/list.txt")
        httpretty.register_uri(httpretty.GET, "http://example.mock/list.txt",
                body='http://example.com/test',
                adding_headers={
                    'Etag': '"test"'
                })

        saved_state, artifacts = self.web.run(None)
        self.assertIn('http://example.com/test', [str(x) for x in artifacts])
        self.assertEqual(saved_state, 'None;"test";200')

    @httpretty.activate
    def test_run_with_200_and_no_state_headers(self):
        httpretty.register_uri(httpretty.HEAD, "http://example.mock/list.txt")
        httpretty.register_uri(httpretty.GET, "http://example.mock/list.txt",
                body='http://example.com/test')

        saved_state, artifacts = self.web.run(None)
        self.assertIn('http://example.com/test', [str(x) for x in artifacts])
        self.assertEqual(saved_state, None)



import io
import sys
import time
import watchdog.events
import watchdog.observers

import threatingestor.extras.queueworker

class FSWatcher(
        watchdog.events.PatternMatchingEventHandler,
        threatingestor.extras.queueworker.QueueWorker):
    """Watch a directory for YARA rule changes.

    Send contents of the changed rule files to the queue.
    """
    # Only match YARA rules.
    patterns = ["*.yar", "*.yara", "*.rule", "*.rules"]

    def process(self, event):
        """Handle a file event."""
        with io.open(event.src_path, 'r') as rule_source:
            rule_content = rule_source.read()
            self.queue.write_one({
                'filename': event.src_path,
                'rules': rule_content,
            })


    def on_modified(self, event):
        self.process(event)


    def on_created(self, event):
        self.process(event)


if __name__ == '__main__':
    observer = watchdog.observers.Observer()
    worker = FSWatcher()
    worker.read_config(sys.argv[1])
    observer.schedule(worker, worker.config['watch_path'])
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()

    observer.join()



import unittest

import threatingestor.sources
import threatingestor.artifacts


class TestSources(unittest.TestCase):

    def setUp(self):
        # patch init
        self.orig_init = threatingestor.sources.Source.__init__
        threatingestor.sources.Source.__init__ = lambda x: None
        self.source = threatingestor.sources.Source()
        self.source.name = 'test'

    def tearDown(self):
        # unpatch init
        threatingestor.sources.Source.__init__ = self.orig_init

    def test_init_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            self.orig_init(self.source, self.source.name)

    def test_run_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            self.source.run(None)

    def test_truncate_length_is_respected(self):
        orig_truncate = threatingestor.sources.TRUNCATE_LENGTH
        threatingestor.sources.TRUNCATE_LENGTH = 15
        content = '123.45.67.89 0 12.33.45.67 890'

        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(artifact_list[0].reference_text, content[:15] + '...')

        threatingestor.sources.TRUNCATE_LENGTH = 8
        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(artifact_list[0].reference_text, content[:8] + '...')

        threatingestor.sources.TRUNCATE_LENGTH = orig_truncate

    def test_content_is_preprocessed(self):
        content = 'blah[.]com/test and test [.] com'

        artifact_list = self.source.process_element(content, 'link')
        self.assertIn('http://blah.com/test', [str(x) for x in artifact_list])
        self.assertIn('blah.com', [str(x) for x in artifact_list])
        self.assertIn('test.com', [str(x) for x in artifact_list])
        self.assertIn('http://test.com', [str(x) for x in artifact_list])
        self.assertEqual(len(artifact_list), 5)

    def test_urls_are_extracted(self):
        content = 'hxxp://someurl.com/test'

        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(str(artifact_list[0]), content.replace('xx', 'tt'))
        self.assertTrue(isinstance(artifact_list[0], threatingestor.artifacts.URL))
        self.assertEqual(str(artifact_list[1]), 'someurl.com')
        self.assertTrue(isinstance(artifact_list[1], threatingestor.artifacts.Domain))

    def test_ips_are_extracted(self):
        content = '232.23.21.12'

        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(str(artifact_list[0]), content)
        self.assertTrue(isinstance(artifact_list[0], threatingestor.artifacts.IPAddress))

    def test_yara_sigs_are_extracted(self):
        content = 'rule testRule { condition: true }'

        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(str(artifact_list[0]), content)
        self.assertTrue(isinstance(artifact_list[0], threatingestor.artifacts.YARASignature))

    def test_urls_matching_reference_link_are_discarded(self):
        content = 'hxxp://someurl.com/test hxxp://noturl.com/test'

        artifact_list = self.source.process_element(content, 'http://someurl.com')
        self.assertIn('http://noturl.com/test', [str(x) for x in artifact_list])
        self.assertIn('noturl.com', [str(x) for x in artifact_list])
        self.assertNotIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertNotIn('someurl.com', [str(x) for x in artifact_list])
        self.assertEqual(len(artifact_list), 3)

    def test_nonobfuscated_urls_are_discarded(self):
        content = 'hxxp://someurl.com/test http://noturl.com/test'

        artifact_list = self.source.process_element(content, 'link')
        self.assertNotIn('http://noturl.com/test', [str(x) for x in artifact_list])
        self.assertNotIn('noturl.com', [str(x) for x in artifact_list])
        self.assertIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertIn('someurl.com', [str(x) for x in artifact_list])
        self.assertEqual(len(artifact_list), 3)

    def test_nonobfuscated_urls_are_included_if_specified(self):
        content = 'hxxp://someurl.com/test http://noturl.com/test'

        artifact_list = self.source.process_element(content, 'link', include_nonobfuscated=True)
        self.assertIn('http://noturl.com/test', [str(x) for x in artifact_list])
        self.assertIn('noturl.com', [str(x) for x in artifact_list])
        self.assertIn('http://someurl.com/test', [str(x) for x in artifact_list])
        self.assertIn('someurl.com', [str(x) for x in artifact_list])
        self.assertEqual(len(artifact_list), 5)

    def test_source_name_is_included_in_artifacts(self):
        content = 'hxxp://someurl.com/test http://noturl.com/test'

        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(len(artifact_list), 3)
        self.assertEqual(artifact_list[0].source_name, 'test')
        self.assertEqual(artifact_list[1].source_name, 'test')

    def test_hashes_are_extracted(self):
        content = """68b329da9893e34099c7d8ad5cb9c940
        $ sha1sum <<< ''
        adc83b19e793491b1c6ea0fd8b46cd9f32e592fc  -
        $ sha256sum <<< ''
        01ba4719c80b6fe911b091a7c05124b64eeece964e09c058ef8f9805daca546b  -
        $ sha512sum <<< ''
        be688838ca8686e5c90689bf2ab585cef1137c999b48c70b92f67a5c34dc15697b5d11c982ed6d71be1e1e7f7b4e0733884aa97c3f7a339a8ed03577cf74be09"""

        artifact_list = self.source.process_element(content, '')
        self.assertIn('68b329da9893e34099c7d8ad5cb9c940', [str(h) for h in artifact_list])
        self.assertIn('adc83b19e793491b1c6ea0fd8b46cd9f32e592fc', [str(h) for h in artifact_list])
        self.assertIn('01ba4719c80b6fe911b091a7c05124b64eeece964e09c058ef8f9805daca546b', [str(h) for h in artifact_list])
        self.assertIn('be688838ca8686e5c90689bf2ab585cef1137c999b48c70b92f67a5c34dc15697b5d11c982ed6d71be1e1e7f7b4e0733884aa97c3f7a339a8ed03577cf74be09',
                      [str(h) for h in artifact_list])
        self.assertTrue(isinstance(artifact_list[0], threatingestor.artifacts.Hash))

    def test_tasks_are_added(self):
        artifact_list = self.source.process_element('', 'test')
        self.assertIn('Manual Task: test', [str(h) for h in artifact_list])
        self.assertTrue(isinstance(artifact_list[0], threatingestor.artifacts.Task))

    def test_uninteresting_ips_are_excluded(self):
        # only the task artifact should get through.
        content = '127.0.0.1'
        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(len(artifact_list), 1)

        content = '192.168.0.1'
        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(len(artifact_list), 1)

        content = '198.51.100.1'
        artifact_list = self.source.process_element(content, 'link')
        self.assertEqual(len(artifact_list), 1)
